@@ -2,7 +2,7 @@ import torch
 import torch.nn as nn
 from abc import abstractclassmethod
 
-from ._util_layers import *
+from ._util_layers import Lambda
 
 
 class KerasLayer(nn.Module):
@@ -65,7 +65,7 @@ class Linear(KerasLayer):
 
 class LayerNorm(KerasLayer):
     def build(self, in_shape):
-        return nn.LayerNorm(in_shape.shape[-1], *self.args, **self.kwargs)
+        return nn.LayerNorm(in_shape[-1], *self.args, **self.kwargs)
 
 
 class GRU(KerasLayer):
