@@ -13,7 +13,7 @@ _losses_dict = OrderedDict({
 
 import torch.nn.functional as F
 
-class CELoss(object):
+class CELoss():
     def __init__(self, label_smoothing=0.0, class_weight=None, reduction='mean'):
         self.label_smoothing = label_smoothing
         self.reduction = reduction
@@ -36,10 +36,12 @@ class CELoss(object):
         return loss
 
 
-def create_loss_by_name(name):
-    name = name.lower()
-    if name not in _losses_dict:
-        raise KeyError(f'Invalid loss name, we support {list(_losses_dict.keys())}.')
-    return _losses_dict[name]()
+def _create_loss(i):
+    if isinstance(i, str):
+        name = i.lower()
+        if name not in _losses_dict:
+            raise KeyError(f'Invalid name, we support {list(_losses_dict.keys())}.')
+        return _losses_dict[name]()
+    return i
 
 __all__ = ['CELoss']
\ No newline at end of file
